"""Functional programming utilities for nonconform.

This module provides decorators, enumerations, and parameter utilities
used throughout the nonconform package.
"""

from .enums import Aggregation

__all__ = [
    "Aggregation",
]
