"""Utility modules for nonconform.

This module provides data handling, functional programming utilities,
and statistical functions used throughout the nonconform package.
"""

from . import data, func, stat

__all__ = [
    "data",
    "func",
    "stat",
]
